@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2005-2016, PyInstaller Development Team.
+# Copyright (c) 2005-2017, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License with exception
 # for distributing bootloader.
@@ -21,6 +21,7 @@ import site
 import subprocess
 import sys
 
+from .log import logger
 
 # Distinguish code for different major Python version.
 is_py2 = sys.version_info[0] == 2
@@ -32,6 +33,7 @@ is_py27 = sys.version_info >= (2, 7) and sys.version_info < (3, 0)
 is_py34 = sys.version_info >= (3, 4)
 is_py35 = sys.version_info >= (3, 5)
 is_py36 = sys.version_info >= (3, 6)
+is_py37 = sys.version_info >= (3, 7)
 
 is_win = sys.platform.startswith('win')
 is_cygwin = sys.platform == 'cygwin'
@@ -42,18 +44,20 @@ is_linux = sys.platform.startswith('linux')
 is_solar = sys.platform.startswith('sun')  # Solaris
 is_aix = sys.platform.startswith('aix')
 is_freebsd = sys.platform.startswith('freebsd')
+is_hpux = sys.platform.startswith('hp-ux')
 
 # Some code parts are similar to several unix platforms
 # (e.g. Linux, Solaris, AIX)
 # Mac OS X is not considered as unix since there are many
 # platform specific details for Mac in PyInstaller.
-is_unix = is_linux or is_solar or is_aix or is_freebsd
+is_unix = is_linux or is_solar or is_aix or is_freebsd or is_hpux
 
 
 # On different platforms is different file for dynamic python library.
 _pyver = sys.version_info[:2]
 if is_win:
-    PYDYLIB_NAMES = {'python%d%d.dll' % _pyver}
+    PYDYLIB_NAMES = {'python%d%d.dll' % _pyver,
+                     'libpython%d.%d.dll' % _pyver}  # For MSYS2 environment
 elif is_cygwin:
     PYDYLIB_NAMES = {'libpython%d%d.dll' % _pyver,
                      'libpython%d%dm.dll' % _pyver,
@@ -61,8 +65,8 @@ elif is_cygwin:
                      'libpython%d.%dm.dll' % _pyver}
 elif is_darwin:
     # libpython%d.%dm.dylib for Conda virtual environment installations
-    PYDYLIB_NAMES = {'Python', '.Python', 
-                     'libpython%d.%d.dylib' % _pyver, 
+    PYDYLIB_NAMES = {'Python', '.Python',
+                     'libpython%d.%d.dylib' % _pyver,
                      'libpython%d.%dm.dylib' % _pyver}
 elif is_aix:
     # Shared libs on AIX are archives with shared object members, thus the ".a" suffix.
@@ -74,6 +78,8 @@ elif is_freebsd:
                      'libpython%d.%dm.so.1' % _pyver,
                      'libpython%d.%d.so.1.0' % _pyver,
                      'libpython%d.%dm.so.1.0' % _pyver}
+elif is_hpux:
+    PYDYLIB_NAMES = {'libpython%d.%d.so' % _pyver}
 elif is_unix:
     # Other *nix platforms.
     # Python 2 .so library on Linux is: libpython2.7.so.1.0
@@ -114,12 +120,17 @@ except NameError:
     stdin_input = input
 
 # Safe repr that always outputs ascii
-
 if is_py2:
     safe_repr = repr
 else:
     safe_repr = ascii
 
+# String types to replace `isinstance(foo, str)`
+# Use `isinstance(foo, string_types)` instead.
+if is_py2:
+    string_types = basestring
+else:
+    string_types = str
 
 # Correct extension ending: 'c' or 'o'
 if __debug__:
@@ -127,18 +138,6 @@ if __debug__:
 else:
     PYCO = 'o'
 
-
-# Obsolete command line options (do not exist anymore).
-_OLD_OPTIONS = [
-    '--upx', '-X',
-    '-K', '--tk',
-    '-C', '--configfile',
-    '--skip-configure',
-    '-o', '--out',
-    '--buildpath',
-    ]
-
-
 # Options for python interpreter when invoked in a subprocess.
 if __debug__:
     # Python started *without* -O
@@ -199,6 +198,30 @@ else:
     modname_tkinter = 'tkinter'
 
 
+# On Windows we require pypiwin32 or pywin32-ctypes
+# -> all pyinstaller modules should use win32api from PyInstaller.compat to
+#    ensure that it can work on MSYS2 (which requires pywin32-ctypes)
+if is_win:
+    try:
+        from PyInstaller.utils.win32 import winutils
+        try:
+            pywintypes = winutils.import_pywin32_module('pywintypes', _is_venv=is_venv)
+            win32api = winutils.import_pywin32_module('win32api', _is_venv=is_venv)
+        except ImportError:
+            try:
+                from win32ctypes.pywin32 import pywintypes
+                from win32ctypes.pywin32 import win32api
+            except ImportError:
+                raise
+    except ImportError:
+        # This environment variable is set by seutp.py
+        # - It's not an error for pywin32 to not be installed at that point
+        if not os.environ.get('PYINSTALLER_NO_PYWIN32_FAILURE'):
+            raise SystemExit('PyInstaller cannot check for assembly dependencies.\n'
+                             'Please install PyWin32 or pywin32-ctypes.\n\n'
+                             'pip install pypiwin32\n')
+
+
 def architecture():
     """
     Returns the bit depth of the python interpreter's architecture as
@@ -503,6 +526,12 @@ def __wrap_python(args, kwargs):
     if is_darwin:
         mapping = {'32bit': '-i386', '64bit': '-x86_64'}
         py_prefix = ['arch', mapping[architecture()]]
+        # Since OS X 10.11 the environment variable DYLD_LIBRARY_PATH is no
+        # more inherited by child processes, so we proactively propagate
+        # the current value using the `-e` option of the `arch` command.
+        if 'DYLD_LIBRARY_PATH' in os.environ:
+            path = os.environ['DYLD_LIBRARY_PATH']
+            py_prefix += ['-e', 'DYLD_LIBRARY_PATH=%s' % path]
         cmdargs = py_prefix + cmdargs
 
     if _PYOPTS:
@@ -573,7 +602,6 @@ def getcwd():
             # Do conversion to ShortPathName really only in case 'cwd' is not
             # ascii only - conversion to unicode type cause this unicode error.
             try:
-                import win32api
                 cwd = win32api.GetShortPathName(cwd)
             except ImportError:
                 pass
@@ -658,34 +686,6 @@ except ImportError:
         return None
 
 
-# Obsolete command line options.
-
-class __obsolete_option:
-    def __init__(self, option_strings, dest, help, **kwargs):
-        self.type = bool
-        self.option_strings = option_strings
-        self.dest = dest
-        self.help = help
-        self.default = self.const = False
-        self.nargs = self.const = self.choices = self.metavar=None
-        self.required = False
-
-    def __call__(parser, namespace, values, opt):
-        return
-        parser.error('%s option does not exist anymore (obsolete).' % opt)
-
-
-def __add_obsolete_options(parser):
-    """
-    Add the obsolete options to a option-parser instance and
-    print error message when they are present.
-    """
-    g = parser.add_argument_group('Obsolete options (not used anymore)')
-    g.add_argument(*_OLD_OPTIONS,
-                   action=__obsolete_option,
-                   help='These options do not exist anymore.')
-
-
 # Site-packages functions - use native function if available.
 if hasattr(site, 'getsitepackages'):
     getsitepackages = site.getsitepackages
@@ -795,6 +795,8 @@ SPECIAL_MODULE_TYPES = {
     'AliasNode',
     'BuiltinModule',
     'RuntimeModule',
+    'RuntimePackage',
+
     # PyInstaller handles scripts differently and not as standard Python modules.
     'Script',
 }
@@ -814,6 +816,13 @@ BAD_MODULE_TYPES = {
     'InvalidSourceModule',
     'InvalidCompiledModule',
     'MissingModule',
+
+    # Runtime modules and packages are technically valid rather than bad, but
+    # exist only in-memory rather than on-disk (typically due to
+    # pre_safe_import_module() hooks) and hence cannot be physically frozen.
+    # For simplicity, these nodes are categorized as bad rather than valid.
+    'RuntimeModule',
+    'RuntimePackage',
 }
 ALL_MODULE_TYPES = VALID_MODULE_TYPES | BAD_MODULE_TYPES
 # TODO Review this mapping to TOC, remove useless entries.
@@ -822,10 +831,11 @@ MODULE_TYPES_TO_TOC_DICT = {
     # Pure modules.
     'AliasNode': 'PYMODULE',
     'Script': 'PYSOURCE',
-    'RuntimeModule': 'PYMODULE',
     'SourceModule': 'PYMODULE',
     'CompiledModule': 'PYMODULE',
     'Package': 'PYMODULE',
+    'FlatPackage': 'PYMODULE',
+    'ArchiveModule': 'PYMODULE',
     # Binary modules.
     'Extension': 'EXTENSION',
     # Special valid modules.
@@ -837,29 +847,19 @@ MODULE_TYPES_TO_TOC_DICT = {
     'InvalidSourceModule': 'invalid',
     'InvalidCompiledModule': 'invalid',
     'MissingModule': 'missing',
+    'RuntimeModule': 'runtime',
+    'RuntimePackage': 'runtime',
     # Other.
-    'FlatPackage': 'PYMODULE',
-    'ArchiveModule': 'PYMODULE',
     'does not occur': 'BINARY',
 }
 
 
 def check_requirements():
     """
-    Verify that all requirements to run PyInstaller are met. Especially
-    PyWin32 is installed on Windows.
+    Verify that all requirements to run PyInstaller are met.
 
     Fail hard if any requirement is not met.
     """
     # Fail hard if Python does not have minimum required version
     if sys.version_info < (3, 3) and sys.version_info[:2] != (2, 7):
         raise SystemExit('PyInstaller requires at least Python 2.7 or 3.3+.')
-
-    if is_win:
-        try:
-            from PyInstaller.utils.win32 import winutils
-            pywintypes = winutils.import_pywin32_module('pywintypes')
-        except ImportError:
-            raise SystemExit('PyInstaller cannot check for assembly dependencies.\n'
-                             'Please install PyWin32.\n\n'
-                             'pip install pypiwin32\n')
